/************************************************************************/
/* Author: 	Hirotaka Miura.						*/
/* Contact: 	Hirotaka.Miura@gmail.com.				*/
/************************************************************************/

/************************************************************************/
/* 05/05/2011: 	Modified.						*/
/* 04/22/2011: 	Previouly modified.					*/
/* 04/22/2011: 	Created.						*/
/************************************************************************/

/************************************************************************/
/* Program: 	clustering_coefficients().				*/
/* Function: 	Calculate overall/average and local clustering coeffs.	*/
/* Notes: 	Refer to Tore Opsahl and Pietro Panzarasa		*/
/* 		Social Networks Volume 31, Issue 2, May 2009, 155-163.	*/
/************************************************************************/
mata:
struct matrix_struct scalar clustering_coefficients(real matrix E, 
  |string scalar nodots){
  /* Declare variables.							*/
  struct matrix_struct scalar z /* Return structure.			*/
  real matrix O /* Outadjacency list.					*/
  real matrix I /* Inadjacency list.					*/
  real matrix W /* Weight outadjacency list.				*/
  real matrix Y /* Weight inadjacency list.				*/
  real matrix L /* Local clustering coefficient matrix.			*/ 
  real matrix C /* Overall/average clustering coefficients.		*/  
  real vector on /* Outneighbor(s) of source vertex.			*/
  real vector in /* Inneighbor(s) of source vertex.			*/
  real vector on2 /* Outneighbors of incoming node to source vertex.	*/
  real vector ow /* Outweights of neighbor(s) of source vertex.		*/
  real vector iw /* Inweights of neighbor(s) of source vertex.		*/
  real vector t /* Source-specific total value of triplets.		*/
  real vector t2 /* Source-specific total value of closed triplets.	*/
  real vector v /* Overall total value of triplets.			*/
  real vector v2 /* Overall total value of closed triplets.		*/
  real scalar j /* Column counter.					*/
  real scalar k /* Column counter.					*/
  real scalar m /* Total nodes in the network.				*/
  real scalar s /* Source vertex.					*/
  real scalar amean /* Arithmetic mean.					*/
  real scalar gmean /* Geometric mean.					*/
  real scalar maxval /* Maximum value.					*/
  real scalar minval /* Minimum value.					*/
  string scalar o /* Display scalar.					*/  
  /**********************************************************************/
  /* Begin program.							*/  
  /**********************************************************************/
  /* Obtain outadjacency list.						*/
  z=adjacency_list(E); O=z.X1; W=z.X2    
  /* Obtain inadjacency list.						*/
  E=(E[.,2],E[.,1],E[.,3]); z=adjacency_list(E); I=z.X1; Y=z.X2 
  /* Obtain total number of nodes in network.				*/
  m=max(rows(O)) 
  /* Initialize storage matrices and vectors.				*/
  L=J(m,4,0); v=J(1,4,0); v2=J(1,4,0)   
  /**********************************************************************/
  /* Inform user of program starting.					*/
  /**********************************************************************/
  o="noisily display "+`""""'; stata(o); displayflush()
  o="Clustering coefficients calculation ("+strofreal(m)+" vertices)"
  o="noisily display "+`"""'+o+`"""'
  stata(o); displayflush()
  /* Display dots unless specified not to.				*/
  if(nodots==""){  
    o="----+--- 1 ---+--- 2 ---+--- 3 ---+--- 4 ---+--- 5"
    o="noisily display "+`"""'+o+`"""'
    stata(o); displayflush()
  }  
  /**********************************************************************/
  /* Loop through each node.						*/
  for(s=1;s<=m;s++){
    /* Exit and continue if source doesn't have at least one outgoing	*/
    /* edge and one incoming edge.					*/
    if(nonmissing(O[s,.])<=1 | nonmissing(I[s,.])<=1){
      if(nonmissing(O[s,.])==1 & nonmissing(I[s,.])==1){
        if(O[s,1]==I[s,1]){
          if(nodots=="") display_status(m,s)
          continue
        }
      }
      else if(nonmissing(O[s,.])<1 | nonmissing(I[s,.])<1){
        if(nodots=="") display_status(m,s)
        continue
      }
    }
    /* Obtain out/inneighbors of source vertex and their weights.	*/
    on=O[s,(1..nonmissing(O[s,.]))]; in=I[s,(1..nonmissing(I[s,.]))]
    ow=W[s,(1..nonmissing(W[s,.]))]; iw=Y[s,(1..nonmissing(Y[s,.]))]
    /***** Calculate total value of triplets.				*/
    /* Define temporary vectors to hold denominator and numerator of	*/
    /* clustering coefficient equation (eq. 2 in OpsPan09).		*/
    t=J(1,4,0); t2=J(1,4,0)
    /* Loop through outneighbors of source vertex.			*/
    for(j=1;j<=cols(on);j++){    
      /* Loop through inneighbors of source vertex.			*/
      for(k=1;k<=cols(in);k++){
	/* Continue to next iteration if out/in edge from same nbhr.	*/
      	if(on[j]==in[k]) continue
      	/* Compute measures.						*/
      	amean=(ow[j]+iw[k])/2
      	gmean=(ow[j]*iw[k])^(1/2)
      	maxval=max((ow[j],iw[k]))
      	minval=min((ow[j],iw[k]))
      	/* Populate total triplet values.				*/
        t[1]=t[1]+amean
        t[2]=t[2]+gmean
        t[3]=t[3]+maxval
        t[4]=t[4]+minval
        /* Obtain outneighbors of incoming node to source vertex.	*/
        on2=O[in[k],(1..nonmissing(O[in[k],.]))]
        /* Populate numerator portion of OpsPan09 eq.2 if triplet.	*/
        if(anyof(on2,on[j])==1){
	  t2[1]=t2[1]+amean
	  t2[2]=t2[2]+gmean
	  t2[3]=t2[3]+maxval
	  t2[4]=t2[4]+minval     
	}
      }
    }
    /* Insert total value of closed triplets divided by total value of	*/
    /* triplets into local clustering coefficient matrix.		*/
    L[s,.]=t2:/t
    /* Update overall values.						*/
    v=v+t; v2=v2+t2
    /* Display status unless specified not to.				*/
    if(nodots=="") display_status(m,s)
  }    
  /***** Overall clustering coefficients.				*/
  /* Divide overall total value of closed triplets with overall value.	*/
  C=J(2,4,.)
  C[1,.]=v2:/v; C[2,.]=mean(L)	
  /***** Notify program is over.					*/
  o="Clustering coefficients calculation completed"
  o="noisily display "+`"""'+o+`"""'
  stata(o); displayflush()  
  /***** Return results.						*/
  z.X1=C; z.X2=L
  return(z)
}
end

  
  
